-----------------------------------------------------------
-- Unsigned integer multiplier using the
--  'shift & add' technique
-----------------------------------------------------------	

library IEEE;
use IEEE.std_logic_1164.all;

entity mult_sum is
   port (
       Clk : in  std_logic;
       X   : in  std_logic_vector (15 downto 0);     -- Input A
       Y   : in  std_logic_vector (15 downto 0);     -- Input B
       P   : out std_logic_vector (2*16-1 downto 0)  -- Result
   );
end mult_sum;

architecture sum_ser_arch of mult_sum is

   component mult_by_1 is
      port (
         SI   : in  std_logic_vector (15 downto 0);
         A    : in  std_logic_vector (15 downto 0);
         BBit : in  std_logic;
         SO   : out std_logic_vector (16 downto 0)
      );
   end component;

   type   part_sum_t is array (0 to 16) of std_logic_vector(16 downto 0);  

   signal part_sum  : part_sum_t;  -- Partial sums, index (i) is entering
                                  --   the mult_by_1 stage multiplying bit X(i)
   signal i_X, i_Y  : std_logic_vector(15     downto 0); -- Inputs registered
   signal i_P       : std_logic_vector(2*16-1 downto 0); -- Multiplication result, pre-register
  
begin

   i_X <= X;
   i_Y <= Y;

   part_sum(0) <= (others => '0');

   iterac: for i in 0 to 15 generate
      norm_stg: mult_by_1 port map (
         SI   => part_sum(i)(16 downto 1),
         A    => i_Y, 
         BBit => i_X(i),
         SO   => part_sum(i+1)
      );
      i_P(i) <= part_sum(i+1)(0);
   end generate;

   i_P(2*16-1 downto 16) <= part_sum(16)(16 downto 1);

   P <= i_P;      

end sum_ser_arch;
